package Acme::CPAN::Patches;

=head1 NAME

Acme::CPAN::Patches - just for testing

=head1 SYNOPSIS

	use Acme::CPAN::Patches;

=head1 DESCRIPTION

=cut

use warnings;
use strict;

our $VERSION = '0.01';

1;


__END__

=head1 AUTHOR

Jozef Kutej

=cut
